/*
pbcY.c
*/

#include "oth.h"
#include "oth_prototype.h"

void copyYpbc(d_complex_t *v)
{
	assert(Pbc.ypbc);

	// Ey
	for (int k = k0Ey - 1; k <= k1Ey + 1; k++) {
	for (int i = i0Ey - 1; i <= i1Ey + 1; i++) {
		//assert(NEY(i,    - 1, k) >= 0 && NEY(i,    - 1, k) < NE);
		//assert(NEY(i, Ny + 0, k) >= 0 && NEY(i, Ny + 0, k) < NE);
		v[NEY(i,    - 1, k)] = v[NEY(i, Ny - 1, k)];
		v[NEY(i, Ny + 0, k)] = v[NEY(i,      0, k)];
	}
	}

	// Ez
	for (int k = k0Ez - 1; k <= k1Ez + 1; k++) {
	for (int i = i0Ez - 1; i <= i1Ez + 1; i++) {
		//assert(NEZ(i,    - 1, k) >= 0 && NEZ(i,    - 1, k) < NE);
		//assert(NEZ(i, Ny + 1, k) >= 0 && NEZ(i, Ny + 1, k) < NE);
		v[NEZ(i,    - 1, k)] = v[NEZ(i, Ny - 1, k)];
		v[NEZ(i, Ny + 1, k)] = v[NEZ(i,      1, k)];
	}
	}

	// Ex
	for (int k = k0Ex - 1; k <= k1Ex + 1; k++) {
	for (int i = i0Ex - 1; i <= i1Ex + 1; i++) {
		//assert(NEX(i,    - 1, k) >= 0 && NEX(i,    - 1, k) < NE);
		//assert(NEX(i, Ny + 1, k) >= 0 && NEX(i, Ny + 1, k) < NE);
		v[NEX(i,    - 1, k)] = v[NEX(i, Ny - 1, k)];
		v[NEX(i, Ny + 1, k)] = v[NEX(i,      1, k)];
	}
	}
}

void factorYpbc(d_complex_t *v)
{
	assert(Pbc.ypbc);
	int64_t n;
	int j;
	const d_complex_t fp = Pbc.fy;
	const d_complex_t fm = d_inv(fp);

	// Ey
	for (int k = k0Ey - 1; k <= k1Ey + 1; k++) {
	for (int i = i0Ey - 1; i <= i1Ey + 1; i++) {
		j =    - 1;
		if ((j >= j0Ey - 1) && (j <= j1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		j = Ny + 0;
		if ((j >= j0Ey - 1) && (j <= j1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ez
	for (int k = k0Ez - 1; k <= k1Ez + 1; k++) {
	for (int i = i0Ez - 1; i <= i1Ez + 1; i++) {
		j =    - 1;
		if ((j >= j0Ez - 1) && (j <= j1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		j = Ny + 1;
		if ((j >= j0Ez - 1) && (j <= j1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ex
	for (int k = k0Ex - 1; k <= k1Ex + 1; k++) {
	for (int i = i0Ex - 1; i <= i1Ex + 1; i++) {
		j =    - 1;
		if ((j >= j0Ex - 1) && (j <= j1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		j = Ny + 1;
		if ((j >= j0Ex - 1) && (j <= j1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}
}
